/**
 * Copyright (C) 2010 Cloudfarming <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *         http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package nl.cloudfarming.client.db.local.jpa;

import java.util.HashMap;
import java.util.Map;

import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.FlushModeType;
import javax.persistence.Persistence;

import nl.cloudfarming.client.db.PersistingModule;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

/**
 * Deze klasse levert een implementatie van het Hibernate Thread Local session
 * register. Ieder actief thread in de JVM kan via dit statische object een
 * specifieke Hibernate session aanvragen en sluiten. Klasse fungeert als een
 * singleton.
 *
 * @author Peter Schuler, mze18215
 */
public class JPASessionLocator {

    /**
     * De referentie naar een Commons Log voor logging.
     */
    private static final Logger log = LoggerFactory.getLogger(JPASessionLocator.class);
    /**
     * Referentie naar de Session factory.
     */
    private EntityManagerFactory entityManagerFactory = null;
    /**
     * De threadlocal instantie voor het opslaan van de gegistreerde sessions.
     */
    private final ThreadLocal<EntityManager> sessionMap = new ThreadLocal<EntityManager>();

    /**
     * Private default constructor voorkomt instantieren van deze klasse.
     */
    public JPASessionLocator(PersistingModule module) {
        try {
            assert module.getModuleName() != null;
            assert module.getPersistenceUnit() != null;

            Map properties = new HashMap();

            StringBuilder sb = new StringBuilder("jdbc:hsqldb:hsql://localhost/cloudfarming");
//            sb.append(System.getProperty("user.dir"));
//            sb.append("/../data/");
//            sb.append(module.getModuleName());
//            sb.append(";create=true");
            properties.put("hibernate.connection.url", sb.toString());
            properties.put("hibernate.connection.driver_class", "org.hsqldb.jdbcDriver");
            properties.put("hibernate.dialect", "org.hibernate.dialect.HSQLDialect");
            properties.put("hibernate.hbm2ddl.auto", "update");
//            properties.put("hibernate.connection.username", module.getModuleName());
//            properties.put("hibernate.connection.password", module.getModuleName());
            properties.put("hibernate.connection.username", "SA");
            properties.put("hibernate.connection.password", "");
            properties.put("hibernate.connection.autocommit", "true");

            entityManagerFactory = Persistence.createEntityManagerFactory(module.getPersistenceUnit(), properties);
        } catch (Exception ex) {
            log.info("Creation of  EntityManagerFactory failed.", ex);
            throw new ExceptionInInitializerError(ex);
        }
    }

    /**
     * Functie retouneert een per aanroepend thread een unieke Hibernate
     * Session. De eerste keer dat een thread deze functie aanroept wordt er een
     * nieuw Hibernate session gestart. Het starten van een transactie wordt
     * overgelaten aan de aanroeper.
     *
     * @return de huidige Hibernate Session voor dit thread.
     */
    public EntityManager getEntityManager() {
        EntityManager retValue = sessionMap.get();

        if (retValue == null) {
            log.debug("New EntityManager linked to  thread "
                    + Thread.currentThread());

//			try {
//				Context ctx = new InitialContext();
//				retValue = (EntityManager) ctx.lookup("java:comp/env/persistence/em");
//			} catch (NamingException e) {
//				throw new IllegalStateException("NamingException",e);
//			}

            retValue = entityManagerFactory.createEntityManager();
            retValue.setFlushMode(FlushModeType.COMMIT);
            sessionMap.set(retValue);
        }

        log.debug("EntityManager linked to thread " + Thread.currentThread()
                + "requested.");

        return retValue;
    }

    /**
     * Functie sluit het actieve Hibernate voor het aanroepende thread. Als er
     * een actieve transactie wordt gevonden, wordt deze ongedaan gemaakt
     * middels een aanroep naar rollback().
     */
    public void closeEntityManager() {
        EntityManager entityManager = sessionMap.get();

        if (entityManager != null) {
            log.debug("EntityManager closed for thread "
                    + Thread.currentThread());

            if (entityManager.getTransaction().isActive()) {
                log.debug("EntityManager linked to thread "
                        + Thread.currentThread()
                        + " contains an open transaction! Rollback executed.");
                entityManager.getTransaction().rollback();
            }

            entityManager.close();
        } else {
            log.debug("There was no EntityManager linked to the thread "
                    + Thread.currentThread() + ".");
        }
        sessionMap.set(null);
    }

    /**
     * Functie geeft aan of dit thread al een actieve HibernateSession heeft.
     *
     * @return true als dit thread over een actieve session beschikt.
     */
    public boolean hasEntityManager() {
        EntityManager session = sessionMap.get();
        return session != null;
    }
}
